export enum FetchStatus {
    Laster = "Laster",
    IkkeHentet = "IkkeHentet",
    Suksess = "Suksess",
    Feil = "Feil",
}

export type Suksess<T> = {
    suksess: true;
    data: T;
    status: FetchStatus.Suksess;
};

export enum FeilType {
    FINNER_IKKE_BRUKER_MED_EPOST = "FINNER_IKKE_BRUKER_MED_EPOST",
    KUNNE_IKKE_HENTE_BRUKERE = "KUNNE_IKKE_HENTE_BRUKERE",
    KUNNE_IKKE_HENTE_KONTRAKTER = "KUNNE_IKKE_HENTE_KONTRAKTER",
    UKJENT_KONTRAKTNUMMER_FORMAT = "UKJENT_KONTRAKTNUMMER_FORMAT",
    UKJENT_FEIL = "UKJENT_FEIL",
    ELRAPP_ROUTES_EXCEPTION = "ELRAPP_ROUTES_EXCEPTION",
    RODE_FEIL = "RODE_FEIL",
}

export type Laster = {
    status: FetchStatus.Laster;
};

export type IkkeHentet = {
    status: FetchStatus.IkkeHentet;
};

export type DataStatus<T> = ApiRespons<T> | Laster | IkkeHentet;

export type Feil = {
    suksess: false;
    feilmelding: string;
    type: FeilType;
    data?: any;
    status: FetchStatus.Feil;
};

export type ApiRespons<T> = Suksess<T> | Feil;

const suksess = <T>(data: T): Suksess<T> => {
    return { suksess: true, data, status: FetchStatus.Suksess };
};
const feil = (feilmelding: string, type: FeilType): Feil => {
    return { suksess: false, feilmelding, type, status: FetchStatus.Feil };
};

const apiResponse = <T>(data: ApiRespons<T>): ApiRespons<T> => {
    if (!data.suksess) {
        return { ...data, status: FetchStatus.Feil };
    }
    return { ...data, status: FetchStatus.Suksess };
};

export const ikkeRevalidate = {
    revalidateIfStale: false,
    revalidateOnFocus: false,
    revalidateOnReconnect: false,
};

export async function fetcher<T>(
    url: string,
    medCredentials: boolean = true,
): Promise<ApiRespons<T>> {
    try {
        const res = await fetch(url, {
            method: "get",
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json",
            },
            credentials: medCredentials ? "include" : "omit",
        });

        if (res.ok) {
            return await res
                .json()
                .then((json) => (erEnApiResponse(json) ? apiResponse(json) : suksess(json)));
        } else {
            console.error(res);
            return feil("Uhåndtert feil oppsto ved henting av data.", FeilType.UKJENT_FEIL);
        }
    } catch (error) {
        console.error(error);
        return feil(`Klarte ikke hente data`, FeilType.UKJENT_FEIL);
    }
}

export async function fetcherPost<T>(
    url: string,
    body: string,
    medCredentials: boolean = true,
): Promise<ApiRespons<T>> {
    try {
        const res = await fetch(url, {
            method: "post",
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json",
            },
            credentials: medCredentials ? "include" : "omit",
            body: body,
        });

        if (res.ok) {
            return res
                .json()
                .then((json) => (erEnApiResponse(json) ? apiResponse(json) : suksess(json)));
        } else {
            console.error(res);
            return feil("Uhåndtert feil oppsto ved henting av data.", FeilType.UKJENT_FEIL);
        }
    } catch (error) {
        console.error(error);
        return feil(`Klarte ikke hente data`, FeilType.UKJENT_FEIL);
    }
}

function erEnApiResponse(json: any): json is ApiRespons<any> {
    return (json as ApiRespons<any>).suksess !== undefined;
}
