import {
    CartesianGrid,
    Label,
    Legend,
    Line,
    LineChart,
    ReferenceArea,
    ReferenceLine,
    ResponsiveContainer,
    Tooltip,
    XAxis,
    YAxis,
} from "recharts";
import {
    formatAxisTid,
    hentGraftittel,
    hentReflinjeBenevning,
    hentStartOgSluttTid,
    hentTicksXaxis,
    hentYAxisLabel,
} from "./grafUtils";
import { CustomTooltip } from "./CustomTooltip";
import React, { ReactNode, useState } from "react";
import { CustomLegend } from "./CustomLegend";
import { Grafdata } from "./Grafer";
import dayjs from "dayjs";
import { GeneriskGrafvalg } from "./GeneriskGrafvalg";
import styled from "styled-components";
import { breakpointNettbrett } from "../../utils/constants";
import { Produksjonsdetaljer } from "../../kartSide/types/produksjonsdetaljer";
import {
    aktivGrafAtom,
    aktivHoverRecordAtom,
    dataKeyAtom,
    grafdataAtom,
    reflinjeIKartAtom,
} from "../jotai/atoms";
import { useAtomValue, useSetAtom } from "jotai";
import { Heading } from "../../components/fellesKomponenter/typografi/Heading";
import { ØktdetaljerLoggEvent, loggEventMedTabellnavn, TabellNavn } from "../../utils/loggFrontend";

interface Props {
    tickArrayYaxis: number[];
    children: ReactNode;
}
export const GeneriskGraf = ({ tickArrayYaxis, children }: Props) => {
    const grafdata = useAtomValue(grafdataAtom);
    const aktivGraf = useAtomValue(aktivGrafAtom);
    const dataKey = useAtomValue(dataKeyAtom);
    const reflinjeIKart = useAtomValue(reflinjeIKartAtom);
    const setAktivHoverRecord = useSetAtom(aktivHoverRecordAtom);

    const { startTid, sluttTid } = hentStartOgSluttTid(grafdata);
    const defaultXaxisDomain = [startTid.valueOf(), sluttTid.valueOf()];
    const defaultXaxisTicks = hentTicksXaxis(startTid, sluttTid);
    const [xaxisDomain, setXaxisDomain] = useState<number[]>(defaultXaxisDomain);
    const [xaxisTicks, setXaxisTicks] = useState<number[]>(defaultXaxisTicks);
    const [seleksjonstidStart, setSeleksjonstidStart] = useState<number | undefined>(undefined);
    const [seleksjonstidSlutt, setSeleksjonstidSlutt] = useState<number | undefined>(undefined);
    const [synligGrafdata, setSynligGrafdata] = useState<Grafdata[]>(grafdata);
    const [visTilbakestillknapp, setVisTilbakestillknapp] = useState(false);

    const yAxisLabel = hentYAxisLabel(aktivGraf, dataKey, grafdata);
    const referanselinjeBenvevning = hentReflinjeBenevning(aktivGraf, dataKey, grafdata);
    const tittel = hentGraftittel(aktivGraf, dataKey);

    const handleSeleksjon = () => {
        if (
            seleksjonstidStart === seleksjonstidSlutt ||
            seleksjonstidSlutt === undefined ||
            seleksjonstidStart === undefined
        ) {
            setSeleksjonstidStart(undefined);
            setSeleksjonstidSlutt(undefined);
            return;
        }

        const domain = [seleksjonstidStart, seleksjonstidSlutt].sort();
        setXaxisDomain(domain);
        setXaxisTicks(hentTicksXaxis(dayjs(domain[0]), dayjs(domain[1])));
        setSeleksjonstidSlutt(undefined);
        setSeleksjonstidStart(undefined);
        setVisTilbakestillknapp(true);
        loggEventMedTabellnavn(TabellNavn.øktdetaljer, ØktdetaljerLoggEvent.zoom);
    };

    const handleReset = () => {
        setXaxisDomain(defaultXaxisDomain);
        setXaxisTicks(defaultXaxisTicks);
        setVisTilbakestillknapp(false);
    };

    return (
        <div>
            <GrafContainer>
                <CustomHeading styleLevel={"4"} as={"h4"}>
                    {tittel}
                </CustomHeading>
                <CustomLegend
                    grafdata={synligGrafdata}
                    setGrafdata={(data: Grafdata[]) => setSynligGrafdata(data)}
                />
                <ResponsiveContainer height={360} minWidth={550}>
                    <LineChart
                        margin={{ top: 50, right: 40, left: 20, bottom: 30 }}
                        style={{ userSelect: "none" }}
                        onMouseDown={(e) =>
                            e !== null && setSeleksjonstidStart(Number(e.activeLabel))
                        }
                        onMouseMove={(e) =>
                            seleksjonstidStart &&
                            e !== null &&
                            setSeleksjonstidSlutt(Number(e.activeLabel))
                        }
                        onMouseUp={handleSeleksjon}
                    >
                        {synligGrafdata
                            ?.filter((data) => data.visible)
                            .map((data, index) => (
                                <Line
                                    key={"linje" + index}
                                    data={data.data}
                                    type="monotone"
                                    dataKey={dataKey}
                                    stroke={data.color}
                                    strokeWidth={2}
                                    dot={false}
                                    activeDot={{ r: 6 }}
                                    name={data.tittel}
                                    isAnimationActive={false}
                                />
                            ))}

                        <CartesianGrid strokeDasharray="3 3" />
                        <XAxis
                            allowDataOverflow
                            dataKey={(produksjonsdetalje: Produksjonsdetaljer) => {
                                return dayjs(produksjonsdetalje.datoLogget).valueOf();
                            }}
                            type="number"
                            tickFormatter={formatAxisTid}
                            allowDuplicatedCategory={false}
                            ticks={xaxisTicks}
                            domain={xaxisDomain}
                            tickCount={20}
                            user-select={"none"}
                        >
                            <Label value="Klokkeslett" offset={-15} position="insideBottom" />
                        </XAxis>
                        <YAxis domain={[0, Math.max(...tickArrayYaxis)]} ticks={tickArrayYaxis}>
                            <Label
                                value={yAxisLabel}
                                offset={15}
                                position="insideBottomLeft"
                                angle={-90}
                            />
                        </YAxis>
                        <Tooltip
                            content={
                                <CustomTooltip
                                    setAktivHoverRecord={setAktivHoverRecord}
                                    type={aktivGraf}
                                />
                            }
                        />

                        {seleksjonstidStart && seleksjonstidSlutt ? (
                            <ReferenceArea
                                x1={seleksjonstidStart}
                                x2={seleksjonstidSlutt}
                                strokeOpacity={0.3}
                            />
                        ) : null}
                        {reflinjeIKart && (
                            <ReferenceLine
                                y={reflinjeIKart}
                                label={{
                                    value: reflinjeIKart + referanselinjeBenvevning,
                                    position: "top",
                                    fontSize: 16,
                                    offset: 10,
                                    fontWeight: "bold",
                                    fill: "red",
                                }}
                                stroke="red"
                                strokeDasharray="3 3"
                                strokeWidth={3}
                            />
                        )}
                        <CartesianGrid strokeDasharray="3 3" />
                    </LineChart>
                </ResponsiveContainer>
            </GrafContainer>
            <GeneriskGrafvalg visTilbakestillknapp={visTilbakestillknapp} handleReset={handleReset}>
                {children}
            </GeneriskGrafvalg>
        </div>
    );
};

const GrafContainer = styled.div`
    @media (max-width: ${breakpointNettbrett}) {
        overflow-x: scroll;
        overflow-y: hidden;
    }
`;

const CustomHeading = styled(Heading)`
    display: flex;
    justify-content: center;
    margin-bottom: 10px;
`;
